# Jan Tappenbeck (OSM-User: Lübeck)
# Additional Functions
# interna: eMail Tobias vom 2010-02-11

package OSM::Tree::Helper ;
#package Helper ;
use strict;
use warnings;

# CHANGELOG:
#
# 2010-02-10  JT     create, XXX_url_obj_info XXX_url_obj_history
# 2010-06-11  TOPEG  make it work, create function _split_object
# 2010-08-10  JT     XXX_url_obj_josm
# 2010-08-11  JT     XXX_url_obj_staticmap, XXX_url_obj_potlatch

use base qw (Exporter AutoLoader);
our $version='0.1';
our @EXPORT = qw (osmtree_url_obj_history osmtree_url_obj_info osmtree_url_obj_josm osmtree_url_obj_potlatch osmtree_url_obj_staticmap) ;

#########################################################
#  start
#########################################################

# to change the server easier
our $server='http://www.openstreetmap.org';
our $server_staticmap='http://dev.openstreetmap.de/staticmap/staticmap.php';
########################################################################
# PRIVATE FUNCTIONS ####################################################
########################################################################

# private sub to check osm_tree_objects
sub _is_osm_tree_obj
{
  my $object=shift;

        # haben wir etwas
  return 0 if(! defined($object) || !ref($object) );

  # ist es sein OSM::Tree Objekt?
  # ist der String kürzer als neun Zeichen?
  # "OSM::Tree" ist genau 9 Zeichen lang
  return 0 if(length(ref($object)) < 9);

        # ich schneide die ersten 9 Zeichen raus
  # und prüfe ob sie "OSM::Tree" entsprechen
  return 0 if(! substr(ref($object), 0, 9) eq 'OSM::Tree' );

  # kennt es die Methoden "id" und "type"?
  return 0 if(!$object->can('id') || !$object->can('type'));

  return 1;
}

# extract in all functions needed informations
sub _split_object
{
  my $object=shift;
  my $message=shift;
  return wantarray?():undef if(! _is_osm_tree_obj($object));

  # eine id sollte auch vorhanden sein
  # eventuell noch prüfen
  # ob es wirklich eine gültige Id (zahl?) ist
  my $id=$object->id();
  return wantarray?():undef if(!$id);

  # ein Typ sollte auch definiert sein
  my $type=lc($object->type());
  return wantarray?():undef if(!$type);

  # die Nachricht sollte ja auch nicht leer "" oder 0 sein, oder?
  $message = $type if(!$message);

  my @data=($object,$message,$id,$type);

  return wantarray?@data:\@data;
}


########################################################################
# PUBLIC FUNCTIONS #####################################################
########################################################################

#########################################################
#  Object-Utils
#########################################################

  sub OSM::Tree::Node::center
  {
    my $self=shift;
    return $self->coordinate(@_);
  }


#########################################################
#  URL
#########################################################


# url_obj_info([object|<message>])
# create a url to the side of osm-object-info
# returns URL to object-info

# first function with more detail info in german for a better understanding
sub osmtree_url_obj_info{ url_obj_info(@_); }
sub url_obj_info
{
  my ($object,$message,$id,$type)=_split_object(@_);

  return undef if(!$object);

  return undef unless( $type eq 'node' || $type eq 'way' || $type eq 'relation' );

  # da vorher alles in Variablen gesteckt wurde wird der Code hier auch lesbarer.
  # qq ist das selbe wie ""
  # nur muss man damit auch in den Klammern die " nicht entwerten
  # ( nur die benutzen einfassenden Zeichen dürfen darin nicht auftauchen )
  # man könnte auch schreiben: qq{...} oder qq[...] oder qq!...! oder qq'...' ...
  return qq(<a href="$server/browse/$type/$id">$message</a>);
}


# url_obj_history([object|<message>])
# create a url to the side of osm-object-history
# returns URL to object-history

sub osmtree_url_obj_history { url_obj_history(@_); }
sub url_obj_history
{
  my ($object,$message,$id,$type)=_split_object(@_);

  return undef if(!$object);

  return undef unless( $type eq 'node' || $type eq 'way' || $type eq 'relation' );

  return qq(<a href="$server/browse/$type/$id/history">$message</a>);
}

# url_obj_josm([object])
# create a url to open objekt in josm-editor
# returns URL to edit in josm-editor

sub osmtree_url_obj_josm { url_obj_josm(@_); }
sub url_obj_josm
  {
	my ($object,$message,$id,$type)=_split_object(splice(@_,0,1));
    my $span  = shift || 0.02;
	
	return undef if(!$object);

	return undef unless( $type eq 'node' || $type eq 'way');
    
    my ($lon,$lat) = $object->center();

	my $string = "<A HREF=\"http://localhost:8111/load_and_zoom?" ;
	$string .= "left=" . ($lon - $span) ;
	$string .= "&right=" . ($lon + $span) ;
	$string .= "&top=" . ($lat + $span) ;
	$string .= "&bottom=" . ($lat - $span) ;
	$string .= "&select=".lc($object->type()) . $object->id() ;
	$string .= "\" target=\"hiddenIframe\">Local JOSM</a>" ;
	return ($string) ;
}
 
# url_obj_potlatch([object|<message>])
# create a url to the side of potlatch
# returns URL to potlatch-edit-website

sub osmtree_url_obj_potlatch { url_obj_potlatch(@_); }
sub url_obj_potlatch
{
	my ($object,$message,$id,$type)=_split_object(splice(@_,0,1));
	my $zoom  = shift || 17;
	
  return undef if(!$object);

  return undef unless( $type eq 'node' || $type eq 'way' || $type eq 'relation' );

  my ($lon,$lat) = $object->center();
  
  return qq(<a href="$server/edit?lat=${lat}&lon=${lon}&zoom=${zoom}">Potlatch</a>);
}


# url_obj_staticmap([object])
# create a url to open objekt in a static map
# returns URL to show in static map

sub osmtree_url_obj_staticmap { url_obj_staticmap(@_); }
sub url_obj_staticmap
  {
	my ($object,$message,$id,$type)=_split_object(splice(@_,0,1));
    my $zoom  = shift || 15;
    my $width = shift || 100;
    my $heigth= shift || 150;
    my $mtype  = shift || "ltblu-pushpin";

  return undef unless( $type eq 'node' || $type eq 'way');

  my ($lon,$lat) = $object->center();

  # hier kann noch die Auswertung mehrerer Punkte angeschlossen werden
  my $clon = $lon;
  my $clat = $lat;
  
  return qq(<img src="$server_staticmap?center=$clat,$clon&zoom=$zoom&size=${width}x${heigth}&markers=$lat,$lon,$mtype">);
  #return "<img src=\"".$server_staticmap".?center=".$clon.",".$clat."&zoom=".$zoom."&size=".$width."x".$height."&markers=".$clon.",".$clat.",".$mtype.">";
}

#########################################################
#  FORMAT
#########################################################

# url_obj_address([object|<message>])
# create a one-line adress
# returns URL to object-history

sub osmtree_format_obj_adress { url_obj_adress(@_); }
sub url_obj_adress
{
  my ($object,$message,$id,$type)=_split_object(@_);

  return undef if(!$object);

  return undef unless( $type eq 'node' || $type eq 'way');

  my $adress='';
  $adress .=      $object->tag('addr:street')->value()      if($object->has_tag('addr:street'));
  $adress .= " ". $object->tag('addr:housenumber')->value() if($object->has_tag('addr:housenumber'));
  $adress .= ", ".$object->tag('addr:postcode')->value()    if($object->has_tag('addr:postcode'));
  $adress .= " ". $object->tag('addr:city')->value()        if($object->has_tag('addr:city'));
  return $adress;
}

1;

