########################################################################
# OSM::Tree:Node

=encoding utf8

=head1 NAME

OSM::Tree::Node -- OSM Node

=head1 SYNOPSIS


=head1 DESCRIPTION

the node-object
derived from L<OSM::Tree::BASE>

=head1 AUTHOR

ToPeG

=head1 METHODS

=over 4

=cut

package OSM::Tree::Node;
use strict;
use warnings;
use base 'OSM::Tree::BASE';

sub _new
{
  my $class=shift;
  my $self={};
  $self->{TREE}=shift;
  $self->{ID}=shift;
  $self->{IGNORE_DELETED}=shift // \do{my $t=0};
  $self->{FORCE_DELETE}=shift  // \do{my $t=0};
  return undef unless(exists($self->{TREE}->{node}->{$self->{ID}}));
  $self->{REF}=$self->{TREE}->{node}->{$self->{ID}};
  bless($self,$class);
  return $self;
}

#=======================================================================

=item longitude()

get the longitude of this node

returns an numer or undef if the XML-tag ist not set

=cut
sub longitude
{ return shift->{REF}->{lon}; }

=item latitude()

get the latitude of this node

returns an numer or undef if the XML-tag ist not set

=cut
sub latitude
{ return shift->{REF}->{lat}; }

=item coordinate()

get the longitude and latitude of this node

returns an Longitude,Latitude

=cut
sub coordinate
{
  my $self=shift();
  return ($self->longitude(),$self->latitude());
}

#=======================================================================
=item way()

get the first way-objects corresponding to this node

see L<OSM::Tree::Way> for more

returns an way-object or undef

=cut
sub way
{
  my $self=shift;
  my @list=$self->way_list();
  return undef unless(@list);
  return shift(@list);
}

=item way_list()

get the way-objects corresponding to this node

see L<OSM::Tree::Way> for more

returns an way-object list

=cut
sub way_list
{
  my $self=shift;
  my @list=();
  for my $way (keys(%{$self->{TREE}->{way}}))
  {
    $way=OSM::Tree::Way->_new($self->{TREE},$way,$self->{IGNORE_DELETED},$self->{FORCE_DELETE});
    if(grep{$_ eq $self->{ID}}$way->node_id_list())
    {
      push(@list,$way);
      next;
    }
  }
  return @list;
}

=item way_id_list()

get the way-ids corresponding to this node

returns an way-id list

=cut
sub way_id_list
{
  my $self=shift;
  my @list=map{$_->id()}$self->way_list();
  return @list;
}

=item way_list_length()

get the length the list of the ways

returns the length of the ways-list

=cut
sub relation_list_length
{ return scalar(shift->way_list()); }

#-----------------------------------------------------------------------
# overwrite

sub to_xml
{
  my $self=shift;
  my $in=shift || '';
  my $ref=$self->{REF};
  $ref->{visible} ||= 'true';
  my @vals;
  for my $name (qw(id version timestamp uid user visible changeset lat lon))
  { push(@vals, qq($name="$ref->{$name}") ) if(defined($ref->{$name})); }
  my $txt=join(' ',@vals);

  if(exists($ref->{action}) && $ref->{action})
  { $txt.=qq( action="$ref->{action}"); }

  my $ret=$in.sprintf(q!<node %s !,$txt);
  if($self->has_tags())
  {
    $ret.=">\n";
    $ret.=$self->tags()->to_xml("$in  ");
    $ret.="$in</node>\n";
  }
  else
  { $ret.="/>\n"}

  return $ret;
}


1;

=back

=head1 See also

L<OSM::Tree::BASE>

L<OSM::Tree::Node>

L<OSM::Tree::Way>

L<OSM::Tree::Relation>

L<OSM::Tree::TagList>

L<OSM::Tree::Polygon>

L<OSM::Tree>

=cut
