########################################################################
# OSM::Tree::XML::Falback

# if no other parser is Avaiable
# this parser would be used

package OSM::Tree::XML::BigFile;
use base 'OSM::Tree::XML::BigFilePP';
use strict;
use warnings;
use BerkeleyDB;
use XML::Parser;

sub parse
{
  my $class=shift;
  my $data=shift;
  my $type=shift;

  my $fh=undef;
  my $db_name='temp.db';

  if($type eq 'file')
  {
    $db_name="$data.db";
    unlink($db_name) if(-f $db_name && -f $data && (stat($db_name))[9] < (stat($data))[9] );
    #open($fh, '<:encoding(UTF-8)', $data) // return undef;
    open($fh, '<', $data) // return undef;
  }
  else
  {
    $fh = new IO::Scalar \$data;
    unlink($db_name) if( -f $db_name );
  }

  my $db=__osm_parse($fh,$db_name);
  close($fh);

  my %hash;
  tie(%hash,__PACKAGE__, $db, 'data');
  return \%hash;
}

sub __osm_parse
{
  my $fh=shift;
  my $db_file=shift;

  my $ret=-f $db_file;

  my $db=BerkeleyDB::Hash->new(-Filename => $db_file, -Flags=>DB_CREATE);
  $db->filter_fetch_value( sub { $_=Encode::decode("UTF-8", $_); });
  $db->filter_store_value( sub { $_=Encode::encode("UTF-8", $_); });

  return $db if($ret);

  my $inside;
  my %pkeys;
  my %all;

  my $parser = new XML::Parser(Handlers => {Start => sub{
      my $parser=shift;
      my $name=shift;
      my $opts={@_};

      $pkeys{$name}++;

      if($name eq 'osm')
      {
        delete($pkeys{$name});
        while(my ($k,$v) = each(%$opts))
        {
              $db->db_put("data/$k",$v);
          $pkeys{$k}++;
        }
      }
      elsif($name eq 'bounds')
      {
        $all{"data/bounds"}=0 unless($all{"data/bounds"});

        while(my ($k,$v) = each(%$opts))
        { $db->db_put("data/bounds/".$all{"data/bounds"}."/$k",$v); }

        $all{"data/bounds"}++;
      }
      elsif($name eq 'bound')
      {
        delete($pkeys{'bound'});
        $pkeys{'bounds'}++;

        $all{"data/bounds"}=0 unless($all{"data/bounds"});

        my @box=split(",",$opts->{box});
        for my $k (qw(minlat minlon maxlat maxlon))
        { $db->db_put("data/bounds/".$all{"data/bounds"}."/$k",shift(@box)); }

        while(my ($k,$v) = each(%$opts))
        {
          next if($k eq 'box');
          $db->db_put("data/bounds/".$all{"data/bounds"}."/$k",$v);
        }

        $all{"data/bounds"}++;
      }
      elsif($name eq 'node')
      {
        my $id=$opts->{id};

        push(@{$all{"data/$name"}},$id);

        my $list="tag";
        while(my ($k,$v) = each(%$opts))
        {
          $list.="\0$k";
          $db->db_put("data/$name/$id/$k",$v);
        }
        $db->db_put("data/$name/$id",$list);

        $inside="data/$name/$id";
      }
      elsif($name eq 'way')
      {
        my $id=$opts->{id};

        push(@{$all{"data/$name"}},$id);

        my $list="nd\0tag";
        while(my ($k,$v) = each(%$opts))
        {
          $list.="\0$k";
          $db->db_put("data/$name/$id/$k",$v);
        }
        $db->db_put("data/$name/$id",$list);

        $inside="data/$name/$id";
      }
      elsif($name eq 'relation')
      {
        my $id=$opts->{id};

        push(@{$all{"data/$name"}},$id);

        my $list="member\0tag";
        while(my ($k,$v) = each(%$opts))
        {
          $list.="\0$k";
          $db->db_put("data/$name/$id/$k",$v);
        }
        $db->db_put("data/$name/$id",$list);

        $inside="data/$name/$id";
      }
      elsif($inside)
      {
        delete($pkeys{$name});
        my $p=$inside;

        if($name eq 'tag')
        {
          my $id=$opts->{k};
          my $v=$opts->{v};
          $db->db_put("$p/tag/$id",$v);

          push(@{$all{"$p/$name"}},$id);
        }
        elsif($name eq 'nd')
        {
          push(@{$all{"$p/$name"}},$opts->{ref});
        }
        elsif($name eq 'member')
        {
          my $id  =$opts->{ref};
          my $type=$opts->{type};
          my $role=$opts->{role};

          push(@{$all{"$p/$name"}},$id);

          $db->db_put("$p/member/$id/type",$type);
          $db->db_put("$p/member/$id/role",$role);
        }
        else
        { die("UNKOWN SUBTYPE $name\n"); }
      }
      #else
      #{ die("UNKOWN TYPE $name\n"); }

      #$inside=undef if($single);
    }});

  $parser->parse($fh);

  $db->db_put("data",join("\0",sort keys(%pkeys)));

  while(my ($k,$v)=each(%all))
  {
    $v=join("\0",@$v) if(ref($v));
    $db->db_put($k,$v);
  }

  #warn join(",",keys(%pkeys))."\n";

  return $db;

}

1;
