########################################################################
# OSM::Tree::XML::Default

# if no other parser is Avaiable
# this parser would be used

package OSM::Tree::XML::Default;
use strict;
use warnings;
use Encode;
use XML::Parser;
use IO::Scalar;

sub parse
{
  my $class=shift;
  my $data=shift;
  my $type=shift;

  my $fh=undef;

  if($type eq 'file')
  { open($fh, '<', $data); }
  else
  { $fh = new IO::Scalar \$data; }

  my $tree=__osm_parse($fh);

  close($fh);
  return $tree;
}

sub __osm_parse
{
  my $fh=shift;

  my $tree={};
  my $inside;

  my $parser = XML::Parser->new(Handlers => {Start => sub{
      my $parser=shift;
      my $name=shift;
      my $opts={@_};

      if($name eq 'osm')
      {
        while(my ($k,$v) = each(%$opts))
        { $tree->{$k}=$v; }
      }
      elsif($name eq 'bounds')
      { push(@{$tree->{bounds}},$opts); }
      elsif($name eq 'bound')
      {
        my @box=split(",",$opts->{box});
        delete($opts->{box});
        for my $k (qw(minlat minlon maxlat maxlon))
        { $opts->{$k}=shift(@box); }

        push(@{$tree->{bounds}},$opts);
      }
      elsif($name eq 'node' || $name eq 'way' || $name eq 'relation')
      {
        my $id=$opts->{id};
        $tree->{$name}->{$id}=$opts;
        $inside=$opts;
      }
      elsif($inside)
      {
        if($name eq 'tag')
        {
          my $k=$opts->{k};
          my $v=$opts->{v};
          $inside->{tag}->{$k}=$v;
        }
        elsif($name eq 'nd')
        { push(@{$inside->{nd}},$opts->{ref}); }
        elsif($name eq 'member')
        {
          my $id =$opts->{ref};
          delete($opts->{ref});
          $inside->{member}->{$id}=$opts;
        }
        else
        { die("UNKOWN SUBTYPE $name\n"); }
      }
      #else
      #{ die("UNKOWN TYPE $name\n"); }

      #$inside=undef if($single);
    }});

  $parser->parse($fh);

  return $tree;
}
1;
